#include "Atom.h"

using namespace std;
using namespace compound;

AtomProperties Atom::atomProperties;

Atom::Atom()
{
    setNumber(0);
    setSymbol("");
    setX(0);
    setY(0);
    setZ(0);
    setCharge(0);
    setIsotope(0);
    setRepeatAtom(false);
    setColor("");
}


void Atom::setNumber(int number)
{
    this->number = number;
}


int Atom::getNumber()
{
    return this->number;
}


void Atom::setSymbol(string symbol)
{
    this->symbol = symbol;
}


string Atom::getSymbol() const
{
    return this->symbol;
}


void Atom::setType(string type)
{
    this->type = type;
}


string Atom::getType() const
{
    return this->type;
}


void Atom::setX(double x)
{
    this->x = x;
}


double Atom::getX() const
{
    return this->x;
}


void Atom::setY(double y)
{
    this->y = y;
}


double Atom::getY() const
{
    return this->y;
}


void Atom::setZ(double z)
{
    this->z = z;
}


double Atom::getZ() const
{
    return this->z;
}


void Atom::setCharge(int charge)
{
    this->charge = charge;
}


int Atom::getCharge() const
{
    return this->charge;
}


void Atom::setIsotope(int isotope)
{
    this->isotope = isotope;
}


int Atom::getIsotope()
{
    return this->isotope;
}


void Atom::setRepeatAtom(bool repeatAtom)
{
    this->repeatAtom = repeatAtom;
}


bool Atom::isRepeatAtom()
{
    return this->repeatAtom;
}


void Atom::setColor(string color)
{
    this->color = color;
}


string Atom::getColor() const
{
    return this->color;
}


void Atom::addBond(Bond * bond)
{
    if(!bond) return;

    for(size_t i=0; i<bonds.size(); i++)
    {
        if(bonds[i] == bond) return;
    }

    bonds.push_back(bond);
}


size_t Atom::getBondNumber() const
{
    return bonds.size();
}


Bond *Atom::getBond(size_t index) const
{
    Bond * ret = 0x00;

    if(index >= 0 && index < getBondNumber())
    {
        ret = bonds[index];
    }

    return ret;
}


int Atom::getHydrogenNumber() const
{
    size_t bondNumber = getBondNumber();
    int bondOrder = 0;

    AtomProperty property = Atom::atomProperties.getProperty(this->getSymbol());
    int orgHandNum, handNum;
    orgHandNum = handNum = property.getHandNumber();

    bool aromatic = false;
    for(size_t i=0; i<bondNumber; i++)
    {
        Bond *bond = getBond(i);

        if(bond->isSingle())
        {
            bondOrder += 1;
        }
        else if(bond->isDouble())
        {
            bondOrder += 2;
        }
        else if(bond->isTriple())
        {
            bondOrder += 3;
        }
        else if(bond->isAromatic())
        {
            bondOrder += 1;
            aromatic = true;
        }
    }

    if(aromatic) ++bondOrder;

    if(getCharge() > 0)
        bondOrder += getCharge();
    else if(getCharge() < 0)
        handNum += getCharge();

    if(orgHandNum > 0)
    {
        int ret = handNum - bondOrder;
        if(ret < 0) ret = 0;
        return ret;
    }
    else
    {
        if(getCharge() < 0)
            bondOrder -= getCharge();
    }

    int ret = 0;
    if(symbol == "" || symbol == "C")
    {
        if(bondOrder <= 4)
            ret = 4 - bondOrder;
    }
    else if(symbol == "N")
    {
        if(bondOrder <= 3)
            ret = 3 - bondOrder;
        else if(bondOrder <= 5)
            ret = 5 - bondOrder;
    }
    else if(symbol == "O")
    {
        if(bondOrder <= 2)
            ret = 2 - bondOrder;
    }
    else if(symbol == "S")
    {
        if(bondOrder <= 2)
            ret = 2 - bondOrder;
        else if(bondOrder <= 4)
            ret = 4 - bondOrder;
        else if(bondOrder <= 6)
            ret = 6 - bondOrder;
    }
    else if(symbol == "Se")
    {
        if(bondOrder <= 2)
            ret = 2 - bondOrder;
        else if(bondOrder <= 4)
            ret = 4 - bondOrder;
        else if(bondOrder <= 6)
            ret = 6 - bondOrder;
    }
    else if(symbol == "P")
    {
        if(bondOrder <= 3)
            ret = 3 - bondOrder;
        else if(bondOrder <= 5)
            ret = 5 - bondOrder;
    }
    else if(symbol == "B")
    {
        if(bondOrder <= 3)
            ret = 3 - bondOrder;
    }
    else if(symbol == "Ge")
    {
        if(bondOrder <= 2)
            ret = 2 - bondOrder;
        else if(bondOrder <= 4)
            ret = 4 - bondOrder;
    }
    else if(symbol == "As")
    {
        if(bondOrder <= 3)
            ret = 3 - bondOrder;
        else if(bondOrder <= 5)
            ret = 5 - bondOrder;
    }
    else if(symbol == "Se")
    {
        if(bondOrder <= 2)
            ret = 2 - bondOrder;
        else if(bondOrder <= 4)
            ret = 4 - bondOrder;
        else if(bondOrder <= 6)
            ret = 6 - bondOrder;
    }
    else if(symbol == "Br")
    {
        if(bondOrder <= 1)
            ret = 1 - bondOrder;
        else if(bondOrder <= 3)
            ret = 3 - bondOrder;
        else if(bondOrder <= 5)
            ret = 5 - bondOrder;
        else if(bondOrder <= 7)
            ret = 7 - bondOrder;
    }
    else if(symbol == "Kr")
    {
        if(bondOrder >= 1 && bondOrder <= 2)
            ret = 2 - bondOrder;
    }
    else if(symbol == "Sn")
    {
        if(bondOrder <= 2)
            ret = 2 - bondOrder;
        else if(bondOrder <= 4)
            ret = 4 - bondOrder;
    }
    else if(symbol == "Sb")
    {
        if(bondOrder <= 3)
            ret = 3 - bondOrder;
        else if(bondOrder <= 5)
            ret = 5 - bondOrder;
    }
    else if(symbol == "Te")
    {
        if(bondOrder <= 2)
            ret = 2 - bondOrder;
        else if(bondOrder <= 4)
            ret = 4 - bondOrder;
        else if(bondOrder <= 6)
            ret = 6 - bondOrder;
    }
    else if(symbol == "I")
    {
        if(bondOrder <= 1)
            ret = 1 - bondOrder;
        else if(bondOrder <= 3)
            ret = 3 - bondOrder;
        else if(bondOrder <= 5)
            ret = 5 - bondOrder;
        else if(bondOrder <= 7)
            ret = 7 - bondOrder;
    }
    else if(symbol == "Xe" && bondOrder > 0)
    {
        if(bondOrder <= 2)
            ret = 2 - bondOrder;
        else if(bondOrder <= 4)
            ret = 4 - bondOrder;
        else if(bondOrder <= 6)
            ret = 6 - bondOrder;
    }

    return ret;
}


bool Atom::isLeaf() const
{
    return (getBondNumber() <= 1) ? true : false;
}

ostream &operator<<(ostream &os, const compound::Atom &atom)
{
    os << atom.getSymbol();
    os << "(" << atom.getX() << ", " << atom.getY() << ", " << atom.getZ() << ")";
    return os;
}

