#ifndef __COMPOUND_ATOM__
#define __COMPOUND_ATOM__

#include <iostream>
#include <string>
#include <vector>
#include "Bond.h"
#include "AtomProperty.h"

namespace compound
{
    class Bond;

    class Atom
    {
        private:
            Atom(const Atom &other);
            Atom &operator=(const Atom &other);

            int number;
            std::string symbol;
            std::string type;   // KEGG Atom type
            double x, y, z;
            int charge, isotope;
            bool repeatAtom;
            std::string color;
            std::vector<compound::Bond *> bonds;

            static compound::AtomProperties atomProperties;

        public:
            Atom();

            void setNumber(int number);
            int getNumber();

            void setSymbol(std::string symbol);
            std::string getSymbol() const;

            void setType(std::string type);
            std::string getType() const;

            void setX(double x);
            double getX() const;

            void setY(double y);
            double getY() const;

            void setZ(double z);
            double getZ() const;

            void setCharge(int charge);
            int getCharge() const;

            void setIsotope(int isotope);
            int getIsotope();

            void setRepeatAtom(bool repeat);
            bool isRepeatAtom();

            void setColor(std::string color);
            std::string getColor() const;

            void addBond(Bond *);
            size_t getBondNumber() const;
            compound::Bond * getBond(size_t index) const;

            int getHydrogenNumber() const;

            bool isLeaf() const;
    };
}


std::ostream &operator<<(std::ostream &os, const compound::Atom &atom);

#endif
