#include "Bond.h"

using namespace std;

const int compound::Bond::TYPE_SINGLE   = 1;
const int compound::Bond::TYPE_DOUBLE   = 2;
const int compound::Bond::TYPE_TRIPLE   = 3;
const int compound::Bond::TYPE_AROMATIC = 4;

const int compound::Bond::STEREO_NONE   = 0;
const int compound::Bond::STEREO_UP     = 1;
const int compound::Bond::STEREO_EITHER = 4;
const int compound::Bond::STEREO_DOWN   = 6;

compound::Bond::Bond()
{
    setFirstAtom(0x00);
    setSecondAtom(0x00);
    setType(TYPE_SINGLE);
    setStereo(STEREO_NONE);
    setColor("");
}


void compound::Bond::setFirstAtom(compound::Atom *first)
{
    this->firstAtom = first;
}


compound::Atom *compound::Bond::getFirstAtom()
{
    return this->firstAtom;
}


void compound::Bond::setSecondAtom(compound::Atom *second)
{
    this->secondAtom = second;
}


compound::Atom *compound::Bond::getSecondAtom()
{
    return this->secondAtom;
}


compound::Atom * compound::Bond::getConnectedAtom(compound::Atom *atom)
{
    Atom * ret = 0x00;

    if(getFirstAtom() == atom)
    {
        ret = getSecondAtom();
    }
    else if(getSecondAtom() == atom)
    {
        ret = getFirstAtom();
    }

    return ret;
}


void compound::Bond::setType(int type)
{
    this->type = type;
}


int compound::Bond::getType()
{
    return this->type;
}


bool compound::Bond::isSingle()
{
    return (getType() == TYPE_SINGLE);
}


bool compound::Bond::isDouble()
{
    return (getType() == TYPE_DOUBLE);
}


bool compound::Bond::isTriple()
{
    return (getType() == TYPE_TRIPLE);
}


bool compound::Bond::isAromatic()
{
    return (getType() == TYPE_AROMATIC);
}


void compound::Bond::setStereo(int stereo)
{
    this->stereo = stereo;
}


int compound::Bond::getStereo()
{
    return this->stereo;
}


void compound::Bond::setColor(string color)
{
    this->color = color;
}


string compound::Bond::getColor()
{
    return this->color;
}


void compound::Bond::setUp()
{
    setType(TYPE_SINGLE);
    setStereo(STEREO_UP);
}


void compound::Bond::setDown()
{
    setType(TYPE_SINGLE);
    setStereo(STEREO_DOWN);
}


bool compound::Bond::isUp()
{
    return (isSingle() && getStereo() == STEREO_UP);
}


bool compound::Bond::isDown()
{
    return (isSingle() && getStereo() == STEREO_DOWN);
}


bool compound::Bond::isUpAndDown()
{
    return (isSingle() && getStereo() == STEREO_EITHER);
}


bool compound::Bond::isRepeatBond()
{
    Atom *first = getFirstAtom();
    Atom *second = getSecondAtom();

    return (first->isRepeatAtom() && second->isRepeatAtom());
}


double compound::Bond::getLength()
{
    Atom *first = getFirstAtom();
    Atom *second = getSecondAtom();

    double x = first->getX() - second->getX();
    double y = first->getY() - second->getY();

    return sqrt((x*x) + (y*y));
}
