#ifndef __COMPOUND_BOND__
#define __COMPOUND_BOND__

#include <math.h>
#include "Atom.h"

namespace compound
{
    class Atom;

    class Bond
    {
        private:
            Bond(const Bond &other);
            Bond &operator=(const Bond &other);

            compound::Atom *firstAtom, *secondAtom;
            int type, stereo;
            std::string color;

            static const int TYPE_SINGLE;
            static const int TYPE_DOUBLE;
            static const int TYPE_TRIPLE;
            static const int TYPE_AROMATIC;

            static const int STEREO_NONE;
            static const int STEREO_UP;
            static const int STEREO_EITHER;
            static const int STEREO_DOWN;

        public:
            Bond();

            void setFirstAtom(compound::Atom *first);
            compound::Atom *getFirstAtom();

            void setSecondAtom(compound::Atom *second);
            compound::Atom *getSecondAtom();

            compound::Atom * getConnectedAtom(compound::Atom *atom);

            void setType(int type);
            int getType();

            bool isSingle();
            bool isDouble();
            bool isTriple();
            bool isAromatic();

            void setStereo(int stereo);
            int getStereo();

            void setColor(std::string color);
            std::string getColor();

            void setUp();
            void setDown();

            bool isUp();
            bool isDown();
            bool isUpAndDown();

            bool isRepeatBond();

            double getLength();
    };
}

#endif
