#include "BracketImage.h"

using namespace std;
using namespace compound;
using namespace image;
using namespace utils;

const double BracketImage::BRACKET_LENGTH = 5.0;

BracketImage::BracketImage(Image *image)
{
    this->image = image;
}


void BracketImage::drawBracket(Bracket *bracket)
{   
    Border *firstBorder = bracket->getFirstBorder();
    Border *secondBorder = bracket->getSecondBorder();
   
    drawFirstBorder(firstBorder);
    drawSecondBorder(secondBorder);

    drawBracketSubscript(bracket);
}


void BracketImage::drawFirstBorder(Border *border)
{   
    PointDbl p1 = border->getFirstPoint();
    PointDbl p2 = border->getSecondPoint();
    
    PointDbl start(p1.getX(), p1.getY());
    PointDbl end(p2.getX(), p2.getY());
    
    image->drawLine(start, end);
    
    PointDbl e1 = start + PointDbl(BRACKET_LENGTH, 0);
    image->drawLine(start, e1);

    PointDbl e2 = end + PointDbl(BRACKET_LENGTH, 0);
    image->drawLine(end, e2);
}

void BracketImage::drawSecondBorder(Border *border)
{
    PointDbl p1 = border->getFirstPoint();
    PointDbl p2 = border->getSecondPoint();

    PointDbl start(p1.getX(), p1.getY());
    PointDbl end(p2.getX(), p2.getY());
        
    image->drawLine(start, end);
        
    PointDbl e1 = start + PointDbl(-BRACKET_LENGTH, 0);
    image->drawLine(start, e1);

    PointDbl e2 = end + PointDbl(-BRACKET_LENGTH, 0);
    image->drawLine(end, e2);
}


void BracketImage::drawBracketSubscript(Bracket *bracket)
{
    string subscript = bracket->getSubscript();

    Border *border = bracket->getSecondBorder();

    PointDbl p1 = border->getFirstPoint();
    PointDbl p2 = border->getSecondPoint();

    PointDbl p;
    if(p1.getY() > p2.getY())
        p = PointDbl(p1.getX(), p1.getY());
    else
        p = PointDbl(p2.getX(), p2.getY());

    image->drawString(p.getX()+2.0, p.getY(), subscript,
            RGB(), ALIGN_LEFT, VALIGN_BOTTOM, LARGE_FONT);
}


RectangleDbl BracketImage::getBracketRegion(Bracket *bracket)
{
    Border *border1 = bracket->getFirstBorder();
    Border *border2 = bracket->getSecondBorder();

    RectangleDbl ret = RectangleDbl(border1->getFirstPoint(), DimensionDbl());

    ret += RectangleDbl(border1->getSecondPoint(), DimensionDbl());
    ret += RectangleDbl(border2->getFirstPoint(), DimensionDbl());
    ret += RectangleDbl(border2->getSecondPoint(), DimensionDbl());

    ret += image->getStringRegion(
                ret.getRightBottom().getX()+2.0, ret.getRightBottom().getY(),
                bracket->getSubscript(), ALIGN_LEFT, VALIGN_BOTTOM, LARGE_FONT);

    return ret;
}
