
#ifndef __COMPOUND_MOLECULE__
#define __COMPOUND_MOLECULE__

#include <iostream>
#include <vector>
#include <float.h>
#include "Atom.h"
#include "Bond.h"
#include "Bracket.h"
#include "SimpleRingChecker.h"
#include "../utils/Dimension.h"

namespace compound
{
    class Molecule
    {
        private:
            Molecule(const Molecule &other);
            Molecule &operator=(const Molecule &other);

            std::vector<Atom *> atoms;
            std::vector<Bond *> bonds;
            std::vector<Bracket *> brackets;

            void moveAtom(double x, double y);
            void moveBracket(double x, double y);

            void scaleAtom(double xTimes, double yTimes);
            void scaleBracket(double xTimes, double yTimes);

            double getAverageAtomSpace() const;

        public:
            Molecule();

            virtual ~Molecule();

            Atom *addNewAtom();
            size_t getAtomNumber() const;
            Atom * getAtom(size_t index) const;

            Bond *addNewBond();
            size_t getBondNumber() const;
            Bond *getBond(size_t index) const;

            Bracket *addNewBracket();
            size_t getBracketNumber() const;
            Bracket *getBracket(size_t index) const;

            void move(double x, double y);

            void scale(double times);
            void scale(double xTimes, double yTimes);

            void setWidth(double width);
            double getWidth() const;

            void setHeight(double height);
            double getHeight() const;

            double getAverageBondLength() const;
            void setBondLength(double length);
            void setDimension(utils::DimensionDbl d);

            std::vector<Ring> getRing();

            bool isColored();
    };
}

#endif
