#ifndef __COMPOUND_MOLECULE_IMAGE__
#define __COMPOUND_MOLECULE_IMAGE__

#include <float.h>
#include <iostream>
#include <sstream>
#include <map>
#include "../image/Image.h"
#include "../utils/Utils.h"
#include "Molecule.h"
#include "BracketImage.h"

namespace compound
{
    enum AtomEditMode
    {
        Normal,
        KCF,
        SymbolAndNumber,
        SymbolOnly
    };

    class MoleculeImage
    {
        private:
            std::map<Bond *, bool> drawnBond;

            double doubleBondSpace;
            double tripleBondSpace;
            double stereoBondBase;

            AtomEditMode atomEditMode;

            bool isPractice;

            image::Image *image;

            utils::RectangleDbl resizeDrawingRectangle(
                    Molecule *mol, utils::RectangleDbl rect);

            void scaleMolecule(Molecule *mol, utils::RectangleDbl rect);
            void scaleBondSpace(Molecule *mol);
            void moveOrigin(Molecule *mol, utils::RectangleDbl rect);

            void drawAtoms(Molecule *mol);
            utils::RectangleDbl drawAtom(Atom *atom);
            utils::RectangleDbl drawIsotope(Atom *atom, utils::RectangleDbl symbolRect);
            utils::RectangleDbl drawHydrogen(Atom *atom, utils::RectangleDbl symbolRect);
            utils::RectangleDbl drawCharge(Atom *atom, utils::RectangleDbl symbolRect);
            utils::RectangleDbl drawString(double x, double y, std::string str,
                    image::RGB, image::Align, image::VAlign, image::FontType);

            bool isDisplayAtomSymbol(Atom *atom);

            std::string getAtomSymbol(Atom *atom);
            int getHydrogenPosition(Atom *atom);

            void drawRings(Molecule *mol);
            void drawRing(Ring ring);
            void drawRingDoubleBond(Ring ring, Bond *bond);
            void drawInnerLine(Ring ring, Bond *bond);
            utils::PointDbl getInnerLineEndPoint(utils::PointDbl center, Atom *atom);

            void drawBonds(Molecule *mol);
            void drawBond(Bond *bond);
            void drawSingleBond(Bond *bond);
            void drawDoubleBond(Bond *bond);
            void drawTripleBond(Bond *bond);
            void drawUpBond(Bond *bond);
            void drawDownBond(Bond *bond);

            void drawBrackets(Molecule *mol);

            void drawTwoLine(Bond *bond, double space);

            utils::PointDbl calcDoubleBondPoint(Bond *bond, double space);
            std::vector<utils::PointDbl> calcStereoBondPoints(Bond *bond);
            std::vector<utils::PointDbl> calcHashedBondPoints(
                            utils::PointDbl point1, utils::PointDbl point2, int split);

        public:
            static const double IMAGE_PADDING;
            static const double AVERAGE_BOND_LENGTH;
            static const double DOUBLE_BOND_SPACE;
            static const double TRIPLE_BOND_SPACE;
            static const double STEREO_BOND_BASE;
            static const double HASHED_BOND_SPACE;

            MoleculeImage(image::Image *image);

            void drawMolecule(Molecule *mol);
            void drawMolecule(Molecule *mol, utils::RectangleDbl rect);

            bool isShowHydrogen();

            void setAtomEditMode(AtomEditMode mode);
            AtomEditMode getAtomEditMode();

            utils::RectangleDbl getMoleculeRegion(const Molecule *mol);
    };
}

#endif

