#include "Ring.h"

using namespace std;
using namespace compound;
using namespace utils;

Ring::Ring()
{
}


Ring::Ring(vector<Bond *> bonds)
{
    this->bonds = bonds;
}


void Ring::addBond(Bond *bond)
{
    bonds.push_back(bond);
}


Bond *Ring::getBond(size_t index) const
{
    Bond *ret = 0x00;

    if(index >= 0 && index < getBondNumber())
        ret = bonds[index];

    return ret;
}


size_t Ring::getBondNumber() const
{
    return bonds.size();
}


bool Ring::isMember(Bond *bond) const
{
    for(size_t i=0; i<getBondNumber(); i++)
    {
        if(getBond(i) == bond)
            return true;
    }

    return false;
}


Bond *Ring::getConnectedBond(Bond *bond, Atom *atom) const
{
    Bond *ret = 0x00;

    for(size_t i=0; i<atom->getBondNumber(); i++)
    {
        Bond *b = atom->getBond(i);

        if(b == bond) continue;

        if(isMember(b))
            ret = b;
    }

    return ret;
}


PointDbl Ring::getCenter()
{
    if(getBondNumber() == 0)
        return PointDbl(0.0, 0.0);

    double minX, minY, maxX, maxY;
    minX = minY = DBL_MAX;
    maxX = maxY = -DBL_MAX;

    for(size_t i=0; i<getBondNumber(); i++)
    {
        Bond *bond = getBond(i);

        Atom *atom1 = bond->getFirstAtom();
        Atom *atom2 = bond->getSecondAtom();

        if(atom1->getX() < minX)
            minX = atom1->getX();

        if(atom1->getX() > maxX)
            maxX = atom1->getX();

        if(atom1->getY() < minY)
            minY = atom1->getY();

        if(atom1->getY() > maxY)
            maxY = atom1->getY();

        if(atom2->getX() < minX)
            minX = atom2->getX();

        if(atom2->getX() > maxX)
            maxX = atom2->getX();

        if(atom2->getY() < minY)
            minY = atom2->getY();

        if(atom2->getY() > maxY)
            maxY = atom2->getY();
    }

    double x = minX + (maxX - minX) / 2;
    double y = minY + (maxY - minY) / 2;
    return PointDbl(x, y);
}


bool Ring::operator==(const Ring &other) const
{
    if(getBondNumber() != other.getBondNumber())
        return false;

    for(size_t i=0; i<getBondNumber(); i++)
    {
        Bond *bond1 = getBond(i);

        bool flag = false;
        for(size_t j=0; j<other.getBondNumber(); j++)
        {
            Bond *bond2 = other.getBond(j);

            if(bond1 == bond2)
            {
                flag = true;
                break;
            }
        }

        if(flag == false)
            return false;
    }

    return true;
}
