#include "SimpleRingChecker.h"

using namespace std;
using namespace compound;

const int SimpleRingChecker::MAX_DEPTH = 6;


SimpleRingChecker::SimpleRingChecker()
{
}

vector<Ring> SimpleRingChecker::getRing(Molecule *mol)
{
    vector<Ring> ret;

    for(size_t i=0; i<mol->getAtomNumber(); i++)
    {
        bonds.clear();
        routedBonds.clear();

        Atom *atom = mol->getAtom(i);

        if(searchRing(atom, atom, NULL, 0))
        {
            Ring ring(bonds);

            bool flag = false;
            for(size_t ringNum=0; ringNum < ret.size(); ringNum++)
            {
                if(ret[ringNum] == ring)
                {
                    flag = true;
                    break;
                }
            }

            if(!flag)
                ret.push_back(ring);
        }

    }

    return ret;
}


bool SimpleRingChecker::searchRing(
        Atom *firstAtom, Atom *currentAtom,
        Bond *routeBond, int depth)
{
    if(depth >= MAX_DEPTH)
        return false;

    for(size_t i=0; i<currentAtom->getBondNumber(); i++)
    {
        Bond *bond = currentAtom->getBond(i);

        if(bond == routeBond)
            continue;

        if(isRoutedBond(bond))
            return false;

        routedBonds.push_back(bond);

        Atom *pair = bond->getConnectedAtom(currentAtom);
        if(pair == firstAtom)
        {
            bonds.push_back(bond);
            return true;
        }
        else
        {
            if(searchRing(firstAtom, pair, bond, depth+1))
            {
                bonds.push_back(bond);
                return true;
            }
            else
            {
                routedBonds.pop_back();
            }
        }
    }

    return false;
}


bool SimpleRingChecker::isRoutedBond(Bond *bond)
{
    if(routedBonds.size() == 0)
        return false;

    vector<Bond *>::iterator itr;
    itr = find(routedBonds.begin(), routedBonds.end(), bond);
    return (bond == *itr) ? true : false;
}
