#ifndef __GLYCAN_GLYCAN__
#define __GLYCAN_GLYCAN__

#include <math.h>
#include <vector>
#include <float.h>

#include "Node.h"
#include "Edge.h"
#include "../compound/Bracket.h"
#include "../utils/Dimension.h"

namespace glycan
{
    class Glycan
    {
        private:
            std::vector<glycan::Node *> nodes;
            std::vector<glycan::Edge *> edges;
            std::vector<compound::Bracket *> brackets;

            void moveNode(double x, double y);
            void moveBracket(double x, double y);

            void scaleNode(double xTimes, double yTimes);
            void scaleBracket(double xTimes, double yTimes);

        public:
            virtual ~Glycan();

            glycan::Node *addNewNode();
            glycan::Node *getNode(size_t index) const;
            size_t getNodeNumber() const;

            glycan::Edge *addNewEdge();
            glycan::Edge *getEdge(size_t index) const;
            size_t getEdgeNumber() const;

            compound::Bracket *addNewBracket();
            compound::Bracket *getBracket(size_t index) const;
            size_t getBracketNumber() const;


            void scale(double times);
            void scale(double xTimes, double yTimes);

            void move(double x, double y);

            void setNodeSpace(double space);
            void setDimension(utils::DimensionDbl);

            void setWidth(double width);
            double getWidth() const;

            void setHeight(double height);
            double getHeight() const;

            double getAverageNodeSpace();
    };
}

#endif
