#include "Image.h"

using namespace std;
using namespace image;
using namespace utils;

Image::Image()
{
    im = NULL;
    init(100, 100, RGB(255,255,255));
}


Image::Image(unsigned int width, unsigned int height, RGB background)
{
    im = NULL;
    init(width, height, background);
}


Image::~Image()
{
    if(im)
        gdImageDestroy(im);
}


void Image::init(unsigned int width, unsigned int height, RGB background)
{
    if(im)
    {
        colors.clear();
        gdImageDestroy(im);
        im = 0x00;
    }

    im = gdImageCreate(width, height);
    //im = gdImageCreateTrueColor(width, height);
    if(im == NULL)
    {
        cerr << "Could not allocate a new image pointer." << endl;
        return;
    }

    // set background color.
    int bg = getColor(background);

    // if use gdImageCreateTrueColor, default background color is black.
    gdImageFilledRectangle(im, 0, 0, width, height, bg);
}


gdImagePtr Image::getGdImagePtr()
{
    return this->im;
}


int Image::getColor(RGB rgb)
{
    string clr = rgb.toString();

    int ret = 0;
    if(colors.count(clr) == 0)
    {
        ret = gdImageColorAllocate(im, rgb.getRed(), rgb.getGreen(), rgb.getBlue());
        colors[rgb.toString()] = ret;
    }
    else
    {
        ret = colors[clr];
    }

    return ret;
}


int Image::getWidth()
{
    return gdImageSX(im);
}


int Image::getHeight()
{
    return gdImageSY(im);
}


bool Image::save(string file, Format format)
{
    FILE *out;
    out = fopen(file.c_str(), "wb");
    if(out == NULL)
    {
        cerr << "Could not open " << file << endl;
        return false;
    }

    if(format == PNG)
        gdImagePng(im, out);
    else
        gdImageGif(im, out);

    fclose(out);

    return true;
}


void Image::drawLine(utils::PointDbl point1, utils::PointDbl point2, RGB rgb)
{
    drawLine(point1.getX(), point1.getY(), point2.getX(), point2.getY(), rgb);
}


void Image::drawLine(double x1, double y1, double x2, double y2, RGB rgb)
{
    int intX1 = (int)round(x1);
    int intY1 = (int)round(y1);
    int intX2 = (int)round(x2);
    int intY2 = (int)round(y2);

    int clr = getColor(rgb);
    gdImageSetAntiAliased(im, clr);
    gdImageLine(im, intX1, intY1, intX2, intY2, gdAntiAliased);
}


void Image::drawRectangle(RectangleDbl rect, RGB rgb)
{
    PointDbl p1 = rect.getLeftTop();
    PointDbl p2 = rect.getRightBottom();

    drawRectangle(p1.getX(), p1.getY(), p2.getX(), p2.getY(), rgb);
}


void Image::drawRectangle(double x1, double y1, double x2, double y2, RGB rgb)
{
    int intX1 = (int)round(x1);
    int intY1 = (int)round(y1);
    int intX2 = (int)round(x2);
    int intY2 = (int)round(y2);

    int clr = getColor(rgb);
    gdImageRectangle(im, intX1, intY1, intX2, intY2, clr);
}


void Image::filledRectangle(RectangleDbl rect, RGB rgb)
{
    PointDbl p1 = rect.getLeftTop();
    PointDbl p2 = rect.getRightBottom();

    filledRectangle(p1.getX(), p1.getY(), p2.getX(), p2.getY(), rgb);
}


void Image::filledRectangle(double x1, double y1, double x2, double y2, RGB rgb)
{
    int intX1 = (int)round(x1);
    int intY1 = (int)round(y1);
    int intX2 = (int)round(x2);
    int intY2 = (int)round(y2);

    int clr = getColor(rgb);

    gdImageFilledRectangle(im, intX1, intY1, intX2, intY2, clr);
}


void Image::drawPolygon(vector<PointDbl > points, RGB rgb)
{
    int clr = getColor(rgb);

    gdPoint gdPoints[points.size()];

    for(size_t i=0; i<points.size(); i++)
    {
        PointDbl p = points[i];
        gdPoints[i].x = (int)round(p.getX());
        gdPoints[i].y = (int)round(p.getY());
    }

    gdImageSetAntiAliased(im, clr);
    gdImagePolygon(im, gdPoints, points.size(), gdAntiAliased);
}


void Image::filledPolygon(vector<PointDbl > points, RGB rgb)
{
    int clr = getColor(rgb);

    gdPoint gdPoints[points.size()];

    for(size_t i=0; i<points.size(); i++)
    {
        PointDbl p = points[i];
        gdPoints[i].x = (int)round(p.getX());
        gdPoints[i].y = (int)round(p.getY());
    }

    gdImageFilledPolygon(im, gdPoints, points.size(), clr);

    // draw AntiAliased line.
    //drawPolygon(points, rgb);
}
