#ifndef __IMAGE_IMAGE__
#define __IMAGE_IMAGE__

#include <iostream>
#include <string>
#include <fstream>
#include <vector>
#include <map>
#include <gd.h>
#include "RGB.h"
#include "../utils/Utils.h"
#include "../utils/Rectangle.h"

namespace image
{
    typedef enum
    {
        SMALL_FONT,
        LARGE_FONT,
        GIANT_FONT,
        TINY_FONT
    } FontType;


    typedef enum
    {
        ALIGN_LEFT,
        ALIGN_CENTER,
        ALIGN_RIGHT
    }Align;


    typedef enum
    {
        VALIGN_TOP,
        VALIGN_CENTER,
        VALIGN_BOTTOM
    } VAlign;


    typedef enum
    {
        PNG,
        GIF
    } Format;


    class Image
    {
        private:
            gdImagePtr im;
            std::map<std::string, int> colors;

        protected:
            int getColor(image::RGB rgb);

        public:
            Image();
            Image(unsigned int width, unsigned int height, RGB background = RGB(255,255,255));
            virtual ~Image();

            void init(unsigned int width, unsigned int height, RGB bg = RGB(255,255,255));

            gdImagePtr getGdImagePtr();

            int getWidth();
            int getHeight();

            bool save(std::string file, Format type=GIF);

            void drawLine(const utils::PointDbl point1, const utils::PointDbl point2, const RGB rgb = RGB());
            void drawLine(double x1, double y1, double x2, double y2, image::RGB rgb = RGB());

            void drawRectangle(const utils::RectangleDbl rect, const RGB rgb = RGB());
            void drawRectangle(double x1, double y1, double x2, double y2, RGB rgb = RGB());

            void filledRectangle(const utils::RectangleDbl rect, const RGB rgb);
            void filledRectangle(double x1, double y1, double x2, double y2, RGB rgb);

            void drawPolygon(std::vector<utils::PointDbl > points, RGB rgb=RGB());
            void filledPolygon(std::vector<utils::PointDbl > points, RGB rgb=RGB());

            virtual utils::RectangleDbl getStringRegion(double x, double y, std::string str,
                    Align align = ALIGN_LEFT, VAlign valign = VALIGN_TOP,
                    FontType type = LARGE_FONT) = 0;

            virtual utils::RectangleDbl drawString(double x, double y, std::string str, RGB rgb = RGB(),
                    Align align = ALIGN_LEFT, VAlign valign = VALIGN_TOP,
                    FontType type = LARGE_FONT) = 0;

    };
}

#endif
