#include "ImageFT.h"

using namespace std;
using namespace image;
using namespace utils;

ImageFT::ImageFT() : Image()
{
    setTinyFontSize(5);
    setSmallFontSize(10);
    setLargeFontSize(15);
    setGiantFontSize(20);
}


ImageFT::ImageFT(unsigned int width, unsigned int height, RGB bg) : Image(width, height, bg)
{
    setTinyFontSize(5);
    setSmallFontSize(10);
    setLargeFontSize(15);
    setGiantFontSize(20);
}


void ImageFT::setFontFile(string fontFile)
{
    // path check.
    fstream fin;
    fin.open(fontFile.c_str(), ios::in);
    if(!fin.is_open())
    {
        cerr << fontFile << " not found." << endl;
        return;
    }

    this->fontFile = fontFile;
}


string ImageFT::getFontFile()
{
    return this->fontFile;
}


void ImageFT::setFontSize(int size)
{
    setTinyFontSize(size * 2 / 3);
    setSmallFontSize(size * 2 / 3);
    setLargeFontSize(size);
    setGiantFontSize(size * 4 / 3);
}


void ImageFT::setSmallFontSize(int size)
{
    this->smallFontSize = size;
}


int ImageFT::getSmallFontSize()
{
    return this->smallFontSize;
}


void ImageFT::setLargeFontSize(int size)
{
    this->largeFontSize = size;
}


int ImageFT::getLargeFontSize()
{
    return this->largeFontSize;
}


void ImageFT::setGiantFontSize(int size)
{
    this->giantFontSize = size;
}


int ImageFT::getGiantFontSize()
{
    return this->giantFontSize;
}


void ImageFT::setTinyFontSize(int size)
{
    this->tinyFontSize = size;
}


int ImageFT::getTinyFontSize()
{
    return this->tinyFontSize;
}


RectangleDbl ImageFT::getStringRegion(
        double x, double y, string str, Align align, VAlign valign, FontType type)
{
    int size = getFtFontSize(type);
    return getStringRegionFT(x, y, str, align, valign, size);
}


RectangleDbl ImageFT::getStringRegionFT(
        double x, double y, string str, Align align, VAlign valign, int size)
{
    char *err, *ttf, *s;
    int brect[8];
    string file;

    memset(brect, 0x00, sizeof(brect));

    ttf = stringToChar(getFontFile());
    s = stringToChar(str);

    // get TrueType Font size.
    err = gdImageStringFT(NULL, brect, 0, ttf, size, 0, 0, 0, s);
    if(err)
    {
        cout << "ImageFT::getStringRegionFT : " << err << endl;
    }

    delete [] s;
    delete [] ttf;

    if(err)
    {
        cerr << err << endl;
        return RectangleDbl();
    }


    double width = brect[2] - brect[0] - 1.0;
    double height = brect[1] - brect[7];

    switch(align)
    {
        case ALIGN_RIGHT:
            x = x - width;
            break;
        case ALIGN_CENTER:
            x = x - (width/2);
            break;
        default:
            break;
    }

    switch(valign)
    {
        case VALIGN_BOTTOM:
            y = y - height;
            break;
        case VALIGN_CENTER:
            y = y - (height/2);
            break;
        case VALIGN_TOP:
            break;
    }

    RectangleDbl ret(x, y, width, height);

    return ret;
}


RectangleDbl ImageFT::drawString(
        double x, double y, string str, RGB rgb, Align align, VAlign valign, FontType type)
{
    int size = getFtFontSize(type);
    return drawStringFT(x, y, str, rgb, align, valign, size);
}


RectangleDbl ImageFT::drawStringFT(
        double x, double y, string str, RGB rgb, Align align, VAlign valign, int size)
{
    gdImagePtr im = getGdImagePtr();
    int clr = getColor(rgb);

    RectangleDbl rect = getStringRegionFT(x, y, str, align, valign, size);
    PointDbl p = rect.getLeftBottom();

    char *ttf = stringToChar(getFontFile());
    char *s = stringToChar(str);

    int brect[8];
    char *err = gdImageStringFT(im, brect, clr, ttf, size, 0,
                        (int)round(p.getX()), (int)round(p.getY()), s);
    if(err)
    {
        cerr << "ImageFT::drawStringFT : " << err << endl;
    }

    delete [] s;
    delete [] ttf;

    return rect;
}


char *ImageFT::stringToChar(string str)
{
    char *ret = new char[str.length() + 1];
    memset(ret, 0x00, (str.length() + 1));
    strncpy(ret, str.c_str(), str.length());

    return ret;
}


int ImageFT::getFtFontSize(FontType type)
{
    int ret;

    switch(type)
    {
        case SMALL_FONT:
            ret = getSmallFontSize();
            break;
        case GIANT_FONT:
            ret = getGiantFontSize();
            break;
        case TINY_FONT:
            ret = getTinyFontSize();
            break;
        default:
            ret = getLargeFontSize();
            break;
    }

    return ret;
}
