#include "RpairKcfReader.h"

using namespace std;
using namespace rpair;
using namespace compound;

bool RpairKcfReader::readKcf(Rpair *rpair, string str)
{
    istringstream in(str);
    return read(rpair, in);
}


bool RpairKcfReader::readKcfFile(Rpair *rpair, string file)
{
    ifstream in;
    in.open(file.c_str(), ios::in);

    if(!in.good())
    {
        cerr << "Could not open " << file << endl;
        return false;
    }

    return read(rpair, in);
}

bool RpairKcfReader::read(Rpair *rpair, istream &in)
{
    string line;

    string mol1, mol2, align;
    string block = "";

    while(getline(in, line))
    {
        if(line.find("ALIGN") == 0)
        {
            block = "align";
            continue;
        }
        else if(line.find("ENTRY1") == 0)
        {
            block = "compound1";
            continue;
        }
        else if(line.find("ENTRY2") == 0)
        {
            block = "compound2";
            continue;
        }
        else if(line.find("///") == 0)
        {
            break;
        }
        else if(line.find(" ") != 0)
        {
            block = "";
        }

        if(block == "align")
        {
            align += line + "\n";
        }
        else if(block == "compound1")
        {
            // format compound kcf to use MoleculeKcfReader.
            //    "  BRACKET   1    -2.6000   -0.4724   -2.6000    0.3379"
            // -> "BRACKET     1    -2.6000   -0.4724   -2.6000    0.3379"
            mol1 += line.substr(2, 10) + "  " + line.substr(12) + "\n";
        }
        else if(block == "compound2")
        {
            mol2 += line.substr(2, 10) + "  " + line.substr(12) + "\n";
        }
    }

    if(mol1 == "")
    {
        cerr << "Illegal KCF format: ENTRY1 block not found." << endl;
        return false;
    }

    if(mol2 == "")
    {
        cerr << "Illegal KCF format: ENTRY2 block not found." << endl;
        return false;
    }

    // read compound blocks.
    MoleculeKcfReader reader;
    if(reader.readKcf(rpair->getFirstMolecule(), mol1) == false)
    {
        return false;
    }

    if(reader.readKcf(rpair->getSecondMolecule(), mol2) == false)
    {
        return false;
    }

    // read align block.
    readAlign(rpair, align);

    return true;
}


void RpairKcfReader::readAlign(Rpair *rpair, string align)
{
    Molecule *firstMol = rpair->getFirstMolecule();
    Molecule *secondMol = rpair->getSecondMolecule();

    string line;
    istringstream in(align);
    while(getline(in, line))
    {
        if(line.length() == 0) continue;

        istringstream in2(line);

        // read align
        // exp) "1  1:P1b  8:P1b  #M1"
        string index, atom1, atom2, rdm;
        in2 >> index >> atom1 >> atom2 >> rdm;

        int atomIndex1, atomIndex2;
        Atom *firstAtom, *secondAtom;
        firstAtom = secondAtom = 0x00;

        if(sscanf(atom1.c_str(), "%d:", &atomIndex1) == 1)
        {
            firstAtom = firstMol->getAtom(atomIndex1 - 1);
        }

        if(sscanf(atom2.c_str(), "%d:", &atomIndex2) == 1)
        {
            secondAtom = secondMol->getAtom(atomIndex2 - 1);
        }

        if(!firstAtom && !secondAtom){
            cerr << "Illegal align block:" << endl << line << endl;
            continue;
        }

        AtomPair pair(firstAtom, secondAtom, rdm);
        rpair->addAtomPair(pair);
    }
}

