#ifndef __UTILS_POINT__
#define __UTILS_POINT__

#include <iostream>

namespace utils
{
    template <class T> class Point
    {
        private:
            T x, y;

        public:
            Point()
            {
                setX(0);
                setY(0);
            };


            Point(T x, T y)
            {
                setX(x);
                setY(y);
            };


            void setX(T x)
            {
                this->x = x;
            };


            T getX() const
            {
                return this->x;
            };


            void setY(T y)
            {
                this->y = y;
            };


            T getY() const
            {
                return this->y;
            };

            bool operator==(const Point<T> &other) const
            {
                if(getX() != other.getX())
                    return false;

                if(getY() != other.getY())
                    return false;

                return true;
            };


            bool operator!=(const Point<T> &other) const
            {
                return !(*this == other);
            };


            Point<T> &operator+=(const Point<T> &other)
            {
                setX(getX() + other.getX());
                setY(getY() + other.getY());
                return *this;
            }


            Point<T> operator+(const Point<T> &other)
            {
                Point<T> ret(*this);
                ret += other;
                return ret;
            }


            Point<T> &operator-=(const Point<T> &other)
            {
                setX(getX() - other.getX());
                setY(getY() - other.getY());
                return *this;
            }


            Point<T> operator-(const Point<T> &other)
            {
                Point<T> ret(*this);
                ret += other;
                return ret;
            }


            Point<T> &operator*=(const T times)
            {
                setX(getX() * times);
                setY(getY() * times);
                return *this;
            }


            Point<T> operator*(const T times)
            {
                Point<T> ret(*this);
                ret *= times;
                return ret;
            }
    };

    typedef Point<int> PointInt;
    typedef Point<double> PointDbl;
}


template <typename T>
std::ostream &operator<<(std::ostream &os, const utils::Point<T> &ins)
{
    std::cout << "Point(x=" << ins.getX() << ", y=" << ins.getY() << ")";
    return os;
};

#endif
