#ifndef __UTILS_RECTANGLE__
#define __UTILS_RECTANGLE__

#include <iostream>
#include "Point.h"
#include "Dimension.h"

namespace utils
{
    template <class T>
    class Rectangle
    {
        private:
            Point<T> leftTop;
            Dimension<T> dimension;

        public:
            Rectangle(T x = 0, T y = 0, T width=0, T height=0)
            {
                Point<T> p(x, y);
                Dimension<T> d(width, height);

                setLeftTop(p);
                setDimension(d);
            };


            Rectangle(Point<T> leftTop, Dimension<T> dimension)
            {
                setLeftTop(leftTop);
                setDimension(dimension);
            };


            void setLeftTop(Point<T> leftTop)
            {
                this->leftTop = leftTop;
            };


            Point<T> getLeftTop() const
            {
                return this->leftTop;
            };


            Point<T> getRightTop() const
            {
                Point<T> ret = getLeftTop();
                ret.setX(ret.getX() + getWidth());
                return ret;
            };


            Point<T> getLeftBottom() const
            {
                Point<T> ret = getLeftTop();
                ret.setY(ret.getY() + getHeight());
                return ret;
            };


            Point<T> getRightBottom() const
            {
                Point<T> ret = getLeftTop();
                ret.setX(ret.getX() + getWidth());
                ret.setY(ret.getY() + getHeight());
                return ret;
            };


            void setWidth(T width)
            {
                this->dimension.setWidth(width);
            };


            T getWidth() const
            {
                return this->dimension.getWidth();
            };


            void setHeight(T height)
            {
                this->dimension.setHeight(height);
            };


            T getHeight() const
            {
                return this->dimension.getHeight();
            };


            void setDimension(Dimension<T> dimension)
            {
                this->dimension = dimension;
            };


            Dimension<T> getDimension() const
            {
                return this->dimension;
            };


            bool operator==(const Rectangle<T> &other) const
            {
                if(getLeftTop() != other.getLeftTop)
                    return false;

                if(getDimension != other.getDimension())
                    return false;

                return true;
            };

            bool operator!=(const Rectangle<T> &other) const
            {
                return !(*this == other);
            };


            Rectangle<T> &operator+=(const Rectangle<T> &other)
            {
                Point<T> leftTop1 = getLeftTop();
                Point<T> leftTop2 = other.getLeftTop();
                Point<T> rightBottom1 = getRightBottom();
                Point<T> rightBottom2 = other.getRightBottom();

                T x1 = (leftTop1.getX() < leftTop2.getX()) ?
                            leftTop1.getX() : leftTop2.getX();

                T y1 = (leftTop1.getY() < leftTop2.getY()) ?
                            leftTop1.getY() : leftTop2.getY();

                T x2 = (rightBottom1.getX() > rightBottom2.getX()) ?
                            rightBottom1.getX() : rightBottom2.getX();

                T y2 = (rightBottom1.getY() > rightBottom2.getY()) ?
                            rightBottom1.getY() : rightBottom2.getY();

                setLeftTop(Point<T>(x1, y1));
                setDimension(Dimension<T>((x2-x1), (y2-y1)));

                return *this;
            }


            Rectangle<T> operator+(const Rectangle<T> &other)
            {
                Rectangle<T> ret(*this);
                ret += other;
                return ret;
            }
    };


    typedef Rectangle<int> RectangleInt;
    typedef Rectangle<double> RectangleDbl;
}


template <typename T>
std::ostream &operator<<(std::ostream &os, const utils::Rectangle<T> &ins)
{
    os << "Rectangle(" << ins.getLeftTop() << ", " << ins.getDimension() << ")";
    return os;
};

#endif
